import * as orm from "typeorm";
import { Field } from "../typings/Field";
import { Relationship } from "../typings/Relationship";
import { SpecialFields } from "../typings/SpecialFields";
import { FieldLike, Operator } from "../typings";
/**
 * DB level join query builder.
 *
 * `JoinQueryBuilder` is a helper class who can perform the DB level join very
 * conveniently and safely.
 *
 * When you are building up an `JoinQueryBuilder` instance from a specific ORM class,
 * the `JoinQueryBuilder` would analyze relationships of the target ORM class in the
 * compilation level. Therefore, if you take any mistake, it would be detected in the
 * IDE directly. Also, when you're finding neighbor entities to join, `JoinQueryBuilder`
 * will hint you through the auto-completion.
 *
 * ![Hints by the Auto Completion](https://raw.githubusercontent.com/samchon/safe-typeorm/master/assets/demonstrations/safe-query-builder.gif)
 *
 * Therefore, you don't need to write any raw SQL join query of the TypeORM more. You
 * don't need to suffer from the run-time join SQL error of the TypeORM more. Just
 * replace the join queries to use this `JoinQueryBuilder` instance and feel free from
 * those pains.
 *
 * Also, if you want to perform the DB level join query not to specifying (a) specific
 * record(s) (through where condition from the neighborhood entity) but to combine data,
 * I recommend you to use {@link AppJoinBuilder} or {@link JsonSelectBuilder} rather
 * than this `JoinQueryBuilder`. It's the reason why the application level join
 * consumes much fewer resource and elapses much shorter time.
 *
 * Type         | DB Join   | App Join
 * :------------|----------:|----------:
 * Records      | 2,258,000 | 165
 * Elapsed Time | 8.07508   | 0.00262
 *
 * @template Mine Target entity to perform the DB join
 * @template Query Target entity of `TypeORM.SelectQueryBuilder`
 * @reference [stackoverflow/join-queries-vs-multiple-queries](https://stackoverflow.com/questions/1067016/join-queries-vs-multiple-queries)
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare class JoinQueryBuilder<Mine extends object, Query extends object = any> {
    private readonly stmt_;
    private readonly mine_;
    private readonly alias_;
    private readonly joined_;
    /**
     * @hidden
     */
    private constructor();
    private _Take_join;
    has(field: SpecialFields<Mine, Relationship.Joinable<any>>): boolean;
    get<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    size(): number;
    empty(): boolean;
    statement(): orm.SelectQueryBuilder<any>;
    private _Decompose_field_like;
    addSelect<Literal extends SpecialFields<Mine, Field>>(fieldLike: FieldLike<Literal>, alias?: string): this;
    addOrderBy<Literal extends SpecialFields<Mine, Field>>(fieldLike: FieldLike<Literal>, order?: "ASC" | "DESC" | undefined, nulls?: "NULLS FIRST" | "NULLS LAST"): JoinQueryBuilder<Mine, Query>;
    addGroupBy<Literal extends SpecialFields<Mine, Field>>(fieldLike: FieldLike<Literal>): JoinQueryBuilder<Mine, Query>;
    andWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, param: Field.MemberType<T, Literal> | null | (() => string)): JoinQueryBuilder<Mine, Query>;
    andWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: OperatorType, param: (OperatorType extends "=" | "!=" | "<>" ? Field.MemberType<T, Literal> | null : Field.MemberType<T, Literal>) | (() => string)): JoinQueryBuilder<Mine, Query>;
    andWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: "IN" | "NOT IN", param: Array<Field.MemberType<T, Literal>> | (() => string)): JoinQueryBuilder<Mine, Query>;
    andWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: "BETWEEN", minimum: Field.MemberType<T, Literal> | (() => string), maximum: Field.MemberType<T, Literal> | (() => string)): JoinQueryBuilder<Mine, Query>;
    orWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, param: Field.MemberType<T, Literal> | null | (() => string)): JoinQueryBuilder<Mine, Query>;
    orWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: OperatorType, param: (OperatorType extends "=" | "!=" | "<>" ? Field.MemberType<T, Literal> | null : Field.MemberType<T, Literal>) | (() => string)): JoinQueryBuilder<Mine, Query>;
    orWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: "IN" | "NOT IN", param: Array<Field.MemberType<T, Literal>> | (() => string)): JoinQueryBuilder<Mine, Query>;
    orWhere<T extends Mine & {
        [P in Literal]: Field;
    }, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(this: JoinQueryBuilder<T, Query>, field: FieldLike<Literal>, operator: "BETWEEN", minimum: Field.MemberType<T, Literal> | (() => string), maximum: Field.MemberType<T, Literal> | (() => string)): JoinQueryBuilder<Mine, Query>;
    private _Where;
    getColumn<Literal extends SpecialFields<Mine, Field>>(field: Literal): string;
    /**
     * Configure an inner join.
     *
     * `JoinQueryBuilder.innerJoin()` is a method configuring a neighbor entity to
     * inner join, by specifying an accessor field of the *Mine* class, who've defined
     * the relationship decorator with the target neighbor entity class.
     *
     * You also can configure another DB joining targets of the target neighbor entity
     * by writing *closure* function or calling additional methods to the returned
     * `JoinQueryBuilder` instance for the target neighbor entity. Of course, such DB
     * join definition chaining would also benefic from the compilation type checking
     * and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    innerJoin<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure an inner join with alias specification.
     *
     * `JoinQueryBuilder.innerJoin()` is a method configuring a neighbor entity to
     * inner join, by specifying an accessor field of the *Mine* class, who've defined
     * the relationship decorator with the target neighbor entity class.
     *
     * You also can configure another DB joining targets of the target neighbor entity
     * by writing *closure* function or calling additional methods to the returned
     * `JoinQueryBuilder` instance for the target neighbor entity. Of course, such DB
     * join definition chaining would also benefic from the compilation type checking
     * and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param alias Alias name specification for the target entity
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    innerJoin<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, alias: string, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure left join.
     *
     * `JoinQueryBuilder.leftJoin()` is a method configuring a neighbor entity to
     * left join, by specifying an accessor field of the *Mine* class, who've defined
     * the relationship decorator with the target neighbor entity class.
     *
     * You also can configure another DB joining targets of the target neighbor entity
     * by writing *closure* function or calling additional methods to the returned
     * `JoinQueryBuilder` instance for the target neighbor entity. Of course, such DB
     * join definition chaining would also benefic from the compilation type checking
     * and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    leftJoin<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure left join with alias specification.
     *
     * `JoinQueryBuilder.leftJoin()` is a method configuring a neighbor entity to
     * left join, by specifying an accessor field of the *Mine* class, who've defined
     * the relationship decorator with the target neighbor entity class.
     *
     * You also can configure another DB joining targets of the target neighbor entity
     * by writing *closure* function or calling additional methods to the returned
     * `JoinQueryBuilder` instance for the target neighbor entity. Of course, such DB
     * join definition chaining would also benefic from the compilation type checking
     * and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param alias Alias name specification for the target entity
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    leftJoin<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, alias: string, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure inner join with mapping.
     *
     * `JoinQueryBuilder.innerJoinAndSelect()` is a method configuring a neighbor entity
     * to inner join, by specifying an accessor field of the *Mine* class, who've
     * defined the relationship decorator with the target neighbor entity class.
     *
     * Furthermore, the `JoinQueryBuilder.innerJoinAndSelect()` method memorizes the
     * target entity data to the relationship decorated accessor. However, if you're
     * using this method not to specifying (a) specific record(s) but to combining data,
     * I recommend you to utilize the {@link AppJsonBuilder} class instead.
     *
     * Anyway, you can configure another DB joining targets of the target neighbor
     * entity by writing *closure* function or calling additional methods to the
     * returned `JoinQueryBuilder` instance for the target neighbor entity. Of course,
     * such DB join definition chaining would also benefic from the compilation type
     * checking and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    innerJoinAndSelect<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure inner join with mapping and alias specification.
     *
     * `JoinQueryBuilder.innerJoinAndSelect()` is a method configuring a neighbor entity
     * to inner join, by specifying an accessor field of the *Mine* class, who've
     * defined the relationship decorator with the target neighbor entity class.
     *
     * Furthermore, the `JoinQueryBuilder.innerJoinAndSelect()` method memorizes the
     * target entity data to the relationship decorated accessor. However, if you're
     * using this method not to specifying (a) specific record(s) but to combining data,
     * I recommend you to utilize the {@link AppJsonBuilder} class instead.
     *
     * Anyway, you can configure another DB joining targets of the target neighbor
     * entity by writing *closure* function or calling additional methods to the
     * returned `JoinQueryBuilder` instance for the target neighbor entity. Of course,
     * such DB join definition chaining would also benefic from the compilation type
     * checking and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param alias Alias name specification for the target entity
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    innerJoinAndSelect<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, alias: string, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure left join with mapping.
     *
     * `JoinQueryBuilder.leftJoinAndSelect()` is a method configuring a neighbor entity
     * to left join, by specifying an accessor field of the *Mine* class, who've
     * defined the relationship decorator with the target neighbor entity class.
     *
     * Furthermore, the `JoinQueryBuilder.leftJoinAndSelect()` method memorizes the
     * target entity data to the relationship decorated accessor. However, if you're
     * using this method not to specifying (a) specific record(s) but to combining data,
     * I recommend you to utilize the {@link AppJsonBuilder} class instead.
     *
     * Anyway, you can configure another DB joining targets of the target neighbor
     * entity by writing *closure* function or calling additional methods to the
     * returned `JoinQueryBuilder` instance for the target neighbor entity. Of course,
     * such DB join definition chaining would also benefic from the compilation type
     * checking and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    leftJoinAndSelect<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    /**
     * Configure left join with mapping and alias specification.
     *
     * `JoinQueryBuilder.leftJoinAndSelect()` is a method configuring a neighbor entity
     * to left join, by specifying an accessor field of the *Mine* class, who've
     * defined the relationship decorator with the target neighbor entity class.
     *
     * Furthermore, the `JoinQueryBuilder.leftJoinAndSelect()` method memorizes the
     * target entity data to the relationship decorated accessor. However, if you're
     * using this method not to specifying (a) specific record(s) but to combining data,
     * I recommend you to utilize the {@link AppJsonBuilder} class instead.
     *
     * Anyway, you can configure another DB joining targets of the target neighbor
     * entity by writing *closure* function or calling additional methods to the
     * returned `JoinQueryBuilder` instance for the target neighbor entity. Of course,
     * such DB join definition chaining would also benefic from the compilation type
     * checking and auto-completion hints, too.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @param alias Alias name specification for the target entity
     * @param closure Closure function for additional DB joins from the target entity
     * @return New `JoinQueryBuilder` instance for the target entity
     */
    leftJoinAndSelect<Field extends SpecialFields<Mine, Relationship.Joinable<any>>>(field: Field, alias: string, closure?: (builder: JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>) => void): JoinQueryBuilder<Relationship.Joinable.TargetType<Mine, Field>, Query>;
    private _Join_atomic;
    private _Join_and_select;
}
