import { Creator } from "../typings/Creator";
import { Field } from "../typings/Field";
import { FieldLike } from "../typings/FieldLike";
import { Operator } from "../typings/Operator";
import { SpecialFields } from "../typings/SpecialFields";
/**
 * Get arguments for the where-equal query.
 *
 * `Model.getWhereArguments()` is a static method returning arguments for the
 * `TypeORM.SelectQueryBuilder.where()` and similar methods.
 *
 * Unlike writing parameters of the `TypeORM.SelectQueryBuilder.where()` or similar methods by
 * yourself manullay, who can cause critical runtime error by a mis-typing error, the
 * `Model.getWhereArguments()` can prevent the mis-typing error in the compilation level.
 *
 * The `Model.getWhereArguments()` even supports the table alias, therefore the table alias
 * addicted column also can take advantage of the compile time validation. In such reason, if you
 * don't ignore error message from the TypeScript error, there can't be any runtime error that is
 * caused by the mis-typing error in the SQL where query level.
 *
 * @template T Type of a model class
 * @template Literal Type of a literal who represents the field that is defined in the *T*
 * @param creator The target model class
 * @param fieldLike Name of the target field in the model class. The field name can contain the
 *                  table alias.
 * @param param A parameter for the where-equal query
 * @return The exact arguments, for the `TypeORM.SelectQueryBuilder.where()` like methods, which
 *         never can be the runtime error
 */
export declare function getWhereArguments<T extends {
    [P in Literal]: Field;
}, Literal extends SpecialFields<T, Field>>(creator: Creator<T>, fieldLike: FieldLike<`${Literal}` | `${string}.${Literal}`>, param: Field.MemberType<T, Literal> | null | (() => string)): [string, Record<string, Field.ValueType<T[Literal]>>];
/**
 * Get arguments for the where query.
 *
 * `Model.getWhereArguments()` is a static method returning arguments for the
 * `TypeORM.SelectQueryBuilder.where()` and similar methods.
 *
 * Unlike writing parameters of the `TypeORM.SelectQueryBuilder.where()` or similar methods by
 * yourself manullay, who can cause critical runtime error by a mis-typing error, the
 * `Model.getWhereArguments()` can prevent the mis-typing error in the compilation level.
 *
 * The `Model.getWhereArguments()` even supports the table alias, therefore the table alias
 * addicted column also can take advantage of the compile time validation. In such reason, if you
 * don't ignore error message from the TypeScript error, there can't be any runtime error that is
 * caused by the mis-typing error in the SQL where query level.
 *
 * @template T Type of a model class
 * @template Literal Type of a literal who represents the field that is defined in the *T*
 * @param creator The target model class
 * @param fieldLike Name of the target field in the model class. The field name can contain the
 *                  table alias.
 * @param operator Operator for the where condition
 * @param param A parameter for the where query
 * @return The exact arguments, for the `TypeORM.SelectQueryBuilder.where()` like methods, which
 *         never can be the runtime error
 */
export declare function getWhereArguments<T extends {
    [P in Literal]: Field;
}, Literal extends SpecialFields<T, Field>, OperatorType extends Operator>(creator: Creator<T>, fieldLike: FieldLike<`${Literal}` | `${string}.${Literal}`>, operator: OperatorType, param: (OperatorType extends "=" | "!=" | "<>" ? Field.MemberType<T, Literal> | null : Field.MemberType<T, Literal>) | (() => string)): [string, Record<string, Field.ValueType<T[Literal]>>];
/**
 * Get arguments for the where-in query.
 *
 * `Model.getWhereArguments()` is a static method returning arguments for the
 * `TypeORM.SelectQueryBuilder.where()` and similar methods.
 *
 * Unlike writing parameters of the `TypeORM.SelectQueryBuilder.where()` or similar methods by
 * yourself manullay, who can cause critical runtime error by a mis-typing error, the
 * `Model.getWhereArguments()` can prevent the mis-typing error in the compilation level.
 *
 * The `Model.getWhereArguments()` even supports the table alias, therefore the table alias
 * addicted column also can take advantage of the compile time validation. In such reason, if you
 * don't ignore error message from the TypeScript error, there can't be any runtime error that is
 * caused by the mis-typing error in the SQL where query level.
 *
 * @template T Type of a model class
 * @template Literal Type of a literal who represents the field that is defined in the *T*
 * @param creator The target model class
 * @param fieldLike Name of the target field in the model class. The field name can contain the
 *                  table alias.
 * @param operator Operator "BETWEEN" for the where condition
 * @param parameters Parameters for the where-in query
 * @return The exact arguments, for the `TypeORM.SelectQueryBuilder.where()` like methods, which
 *         never can be the runtime error
 */
export declare function getWhereArguments<T extends {
    [P in Literal]: Field;
}, Literal extends SpecialFields<T, Field>>(creator: Creator<T>, fieldLike: FieldLike<`${Literal}` | `${string}.${Literal}`>, operator: "IN" | "NOT IN", parameters: Array<Field.MemberType<T, Literal>> | (() => string)): [
    string,
    Record<string, [Field.ValueType<T[Literal]>, Field.ValueType<T[Literal]>]>
];
/**
 * Get arguments for the where-between query.
 *
 * `Model.getWhereArguments()` is a static method returning arguments for the
 * `TypeORM.SelectQueryBuilder.where()` and similar methods.
 *
 * Unlike writing parameters of the `TypeORM.SelectQueryBuilder.where()` or similar methods by
 * yourself manullay, who can cause critical runtime error by a mis-typing error, the
 * `Model.getWhereArguments()` can prevent the mis-typing error in the compilation level.
 *
 * The `Model.getWhereArguments()` even supports the table alias, therefore the table alias
 * addicted column also can take advantage of the compile time validation. In such reason, if you
 * don't ignore error message from the TypeScript error, there can't be any runtime error that is
 * caused by the mis-typing error in the SQL where query level.
 *
 * @template T Type of a model class
 * @template Literal Type of a literal who represents the field that is defined in the *T*
 * @param creator The target model class
 * @param fieldLike Name of the target field in the model class. The field name can contain the
 *                  table alias.
 * @param operator Operator "BETWEEN" for the where condition
 * @param minimum Minimum parameter of the between range
 * @param maximum Maximum parameter of the between range
 * @return The exact arguments, for the `TypeORM.SelectQueryBuilder.where()` like methods, which
 *         never can be the runtime error
 */
export declare function getWhereArguments<T extends {
    [P in Literal]: Field;
}, Literal extends SpecialFields<T, Field>>(creator: Creator<T>, fieldLike: FieldLike<`${Literal}` | `${string}.${Literal}`>, operator: "BETWEEN", minimum: Field.MemberType<T, Literal> | (() => string), maximum: Field.MemberType<T, Literal> | (() => string)): [string, Record<string, Array<Field.ValueType<T[Literal]>>>];
